package com.sd.business.service.inventory.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.in.po.InOutStorageDetails;
import com.sd.business.entity.inventory.bo.InOutFun;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.po.BalancePriceNum;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.po.InventoryBackup;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.mapper.inventory.InventoryMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageDetailsService;
import com.sd.business.service.in.impl.InOutStorageServiceImpl;
import com.sd.business.service.inventory.InventoryService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Slf4j
@Service
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private InOutStorageDetailsService inOutStorageDetailsService;

    @Override
    public Page<InventoryVo> getPage(InventorySelectDto dto, String tableName) {
        IWrapper<Inventory> wrapper = getWrapper();

        wrapper.orderByDesc("i", Inventory::getId);
        wrapper.eq("b", Bom::getBomClassifyId, dto.getBomClassifyId());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.eq("b", Bom::getSpecies, dto.getBomSpecies());
        wrapper.eq("b", Bom::getChromatophore, dto.getBomChromatophore());
        wrapper.eq("b", Bom::getEmbossingProcess, dto.getBomEmbossingProcess());
        wrapper.eq("b", Bom::getFrontGrain, dto.getBomFrontGrain());
        wrapper.eq("b", Bom::getReverseGrain, dto.getBomReverseGrain());
        wrapper.like("bs", BomSpec::getColour, dto.getBomSpecColour());
        wrapper.like("bs", BomSpec::getLength, dto.getBomSpecLength());
        wrapper.like("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("i", Inventory::getWarehouseId, dto.getWarehouseId());
        wrapper.eq("i", Inventory::getDepartmentId, dto.getDepartmentId());
        wrapper.eq("i", InventoryBackup::getBackupDate, dto.getBackupDate());

        Page<InventoryVo> page = this.baseMapper.getPage(dto.getPage(), wrapper, tableName);
        return page;
    }
    @Override
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto) {
        return baseMapper.getQuantityByWarehouse(dto);
    }

    @Override
    public List<QuantityByDepartmentVo> getQuantityByDepartment() {
        return baseMapper.getQuantityByDepartment();
    }

    @Override
    public void in(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {
        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);
            Map<Long, Inventory> map = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            for (InOutFun inOutFun : list) {
                Long bomSpecId = inOutFun.getBomSpecId();
                BigDecimal quantity = inOutFun.getQuantity();
                BigDecimal unitPrice = InOutStorageServiceImpl.UNITPRICE;
                Inventory inventory = map.get(bomSpecId);

                if (inventory != null) {
                    inventory.setQuantity(inventory.getQuantity().add(quantity));
                } else {
                    inventory = new Inventory();
                    inventory.setWarehouseId(warehouseId);
                    inventory.setDepartmentId(departmentId);
                    inventory.setBomSpecId(bomSpecId);
                    inventory.setQuantity(quantity);
                    inventoryList.add(inventory);
                    map.put(bomSpecId, inventory);
                }
                Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap = getInOutStorageDetailsMap(warehouseId, bomSpecIdList);

                //获取结存数量
                BigDecimal sumQuantity = inOutStorageDetailsMap.getOrDefault(inOutFun.getBomSpecId(), Collections.emptyList())
                        .stream().map(InOutStorageDetails::getQuantity)
                        .filter(item -> item.compareTo(BigDecimal.ZERO) > 0)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                BigDecimal sumPrice = inOutStorageDetailsMap.getOrDefault(inOutFun.getBomSpecId(), Collections.emptyList()).stream()
                        .filter(item -> item.getQuantity().compareTo(BigDecimal.ZERO) > 0)
                        .map(item -> item.getQuantity().multiply(item.getInUnitPrice()))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                // 计算最新的结存数量和结存金额
                sumQuantity = sumQuantity.add(quantity);
                BigDecimal balancep = quantity.multiply(unitPrice);
                sumPrice = sumPrice.add(balancep);
                BigDecimal balanceUnitPrice = sumPrice.divide(sumQuantity, 2, BigDecimal.ROUND_HALF_UP);
                inventory.setBalanceUnitPrice(balanceUnitPrice);
            }
            saveOrUpdateBatch(inventoryList);
        }
    }

    @Override
    public void out(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {

        //存储一条数据出库的数量
        Map<Long, BigDecimal> map = list.stream()
                .collect(Collectors.toMap(InOutFun::getBomSpecId, InOutFun::getQuantity));

        List<Long> bomSpecIdList = new ArrayList<>(map.keySet());

        synchronized (this) {

            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);

            // 获取bom规格id和结存明细列表map
            Map<Long, List<InOutStorageDetails>> inOutStorageDetailsMap = getInOutStorageDetailsMap(warehouseId, bomSpecIdList);

            // 存储需要修改的入库记录
            List<InOutStorageDetails> updateList = new ArrayList<>();

            // 查询库存中是否存在bom
            for (Inventory inventory : inventoryList) {

                Long bomSpecId = inventory.getBomSpecId();

                if (!bomSpecIdList.contains(bomSpecId)) {
                    BomSpec bomSpec = bomSpecService.getById(bomSpecId);
                    if (bomSpec == null) {
                        throw new ServiceException("未知bom规格id：" + bomSpecId);
                    }
                    throw new ServiceException("出库失败,品名为：" + bomSpec.getName() + "的bom库存不足");
                }

                List<InOutStorageDetails> outStorageDetails = inOutStorageDetailsMap.getOrDefault(bomSpecId, Collections.emptyList());
                // 循环次数
                int j = 0;

                // 需要出库数量
                BigDecimal quantity = map.get(bomSpecId);

                do {
                    if (outStorageDetails.size() == j) {
                        throw new ServiceException("数据异常，无法计算库存单价!!");
                    }
                    // 获取这条记录剩余的数量
                    InOutStorageDetails inOutStorageDetails = outStorageDetails.get(j);

                    // 单条可出库数量
                    BigDecimal tempQuantity = inOutStorageDetails.getQuantity();

                    if (tempQuantity.compareTo(quantity) > 0) {
                        inOutStorageDetails.setQuantity(tempQuantity.subtract(quantity));
                        quantity = BigDecimal.ZERO;
                    } else if (tempQuantity.compareTo(quantity) < 0) {
                        inOutStorageDetails.setQuantity(BigDecimal.ZERO);
                        quantity = tempQuantity.subtract(quantity).abs();
                    } else {
                        inOutStorageDetails.setQuantity(BigDecimal.ZERO);
                        quantity = BigDecimal.ZERO;
                    }
                    updateList.add(inOutStorageDetails);
                    j++;

                } while (quantity.compareTo(BigDecimal.ZERO) > 0);

                // 库存总数量
                BigDecimal totalInventory = outStorageDetails.stream()
                        .map(InOutStorageDetails::getQuantity)
                        .filter(item -> item.compareTo(BigDecimal.ZERO) > 0)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                // 库存总金额
                BigDecimal totalInventoryAmount = outStorageDetails.stream()
                        .filter(item -> item.getQuantity().compareTo(BigDecimal.ZERO) > 0)
                        .map(item -> item.getQuantity().multiply(item.getInUnitPrice()))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                BigDecimal balanceUnitPrice;
                // 结存单价
                if (totalInventory.compareTo(BigDecimal.ZERO) == 0) {
                    balanceUnitPrice = BigDecimal.ZERO;
                } else {
                    balanceUnitPrice = totalInventoryAmount.divide(totalInventory, 2, RoundingMode.HALF_UP);
                }
                // 赋值结存单价
                inventory.setBalanceUnitPrice(balanceUnitPrice);
                inventory.setQuantity(totalInventory);
            }

            // 修改计算结存单价的出入库明细表
            inOutStorageDetailsService.updateBatchById(updateList);

            // 修改库存数量以及结存单价
            updateBatchById(inventoryList);
        }
    }
    private List<Inventory> getInventoryList(Long departmentId, Long warehouseId, List<Long> bomSpecIdList) {
        return list(q -> q
                .eq(Inventory::getDepartmentId, departmentId)
                .eq(Inventory::getWarehouseId, warehouseId)
                .in(Inventory::getBomSpecId, bomSpecIdList)
        );
    }
    private Map<Long, List<InOutStorageDetails>> getInOutStorageDetailsMap(Long warehouseId, List<Long> bomSpecIdList) {
        List<InOutStorageDetails> list = inOutStorageDetailsService.list(q -> q
                .eq(InOutStorageDetails::getWarehouseId, warehouseId)
                .in(InOutStorageDetails::getBomSpecId, bomSpecIdList)
                .gt(InOutStorageDetails::getQuantity, BigDecimal.ZERO)
        );
        return list.stream().collect(Collectors.groupingBy(InOutStorageDetails::getBomSpecId));
    }

}
