package com.sd.business.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.purchase.dto.PurchaseDto;
import com.sd.business.entity.purchase.dto.PurchaseSelectDto;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.purchase.vo.PurchaseBomVo;
import com.sd.business.entity.purchase.vo.PurchaseVo;
import com.sd.business.entity.supplier.po.Supplier;
import com.sd.business.mapper.purchase.PurchaseMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.service.purchase.PurchaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 采购合同 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class PurchaseServiceImpl extends ServiceImpl<PurchaseMapper, Purchase> implements PurchaseService {

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<PurchaseVo> getPage(PurchaseSelectDto dto) {
        IWrapper<Purchase> wrapper = getWrapper();
        wrapper.like("p", Purchase::getCode, dto.getCode());
        wrapper.eq("p", Purchase::getFlowStatus, dto.getFlowStatus());
        wrapper.like("s", Supplier::getName, dto.getSupplierName());
        wrapper.orderByDesc("p", Purchase::getId);

        // 查询采购单数据
        Page<PurchaseVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);

        List<PurchaseVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 赋值采购明细
        List<Long> idList = records.stream().map(PurchaseVo::getId).collect(Collectors.toList());
        List<PurchaseBom> list = purchaseBomService.list(q -> q.in(PurchaseBom::getPurchaseId, idList));
        List<PurchaseBomVo> purchaseBomList = BeanUtil.copyToList(list, PurchaseBomVo.class);
        bomSpecService.attributeAssign(purchaseBomList, PurchaseBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
            item.setBomSpecColour(bomSpec.getColour());
            item.setBomSpecLength(bomSpec.getLength());
            item.setBomSpecWidth(bomSpec.getWidth());
            item.setBomSpecHeight(bomSpec.getHeight());
        });

        Map<Long, List<PurchaseBomVo>> map = purchaseBomList.stream().collect(Collectors.groupingBy(PurchaseBom::getPurchaseId));
        for (PurchaseVo record : records) {
            record.setPurchaseBomList(map.getOrDefault(record.getId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public PurchaseVo detail(Long id) {
        Purchase Purchase = this.getById(id);
        PurchaseVo result = BeanUtil.toBean(Purchase, PurchaseVo.class);
        return result;
    }

    @Override
    public void add(PurchaseDto purchaseDto) {
        this.save(purchaseDto);
    }

    @Override
    public void edit(PurchaseDto purchaseDto) {
        this.updateById(purchaseDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
